;..................................................................................
;Call convention:
; Presence test : In    AH = 12h
;                 Out   AX = 2100h
;                       No actions taken 
;
;                  
; Put Pixel     : In    AH = 00h
;                       AL = color (in low-order nibble) 
;                       CX = Y coordinate
;                       DX = X coordinate
;                 Out   A pixel is drawn on screen
;
; xor Put Pixel : In    AH = 01h
;                       AL = color (in low-order nibble) 
;                       CX = Y coordinate
;                       DX = X coordinate
;                 Out   A pixel is drawn on screen in xor_put mode
;
; Get Pixel     : In    AH = 02h
;                       CX = Y coordinate
;                       DX = X coordinate
;                 Out   AX = Pixel color
;
; Box           : In    AH = 03h
;                       AL = Color
;                       CX = Upper left corner Y1
;                       DX = Upper left corner X1 
;                       SI = X2
;                       DI = Y2
;                         
;
;..................................................................................

CODE segment 
      ORG    100h
      assume CS:CODE

 start:  JMP   install

      CR        EQU  0Dh
      LF        EQU  0Ah
 
      Welcome   db CR, LF
		db 'Graphics Booster 1.5'
		db CR, LF
		db '(c) Copyright 1996, E. Elachtar, I. Chihi, I. Romdhani', LF, CR
		db 'National Computer Science School','$' 

 
      AlreadyIn db CR, LF, 'Graphics Booster is already installed !', LF, CR, '$'
 
      InstSucc  db LF, CR, 'Successfuly installed', LF, CR, '$'

      X1        dw 0
      X2        dw 0
      Y1        dw 0
      Y2        dw 0
      Color     db 0

   WinPalette   db 0, 4, 2, 6, 1, 5, 3, 8, 7, 12, 10, 14, 9, 13, 11, 15
 
 

      TSRint32  proc far       ; .The new entry to INT 32

		  JMP    CheckEntry
	  PutPixelProc   Proc NEAR 
			 PUSH   CX
			 PUSH   ES
			 PUSH   DX

			 MOV    AX, CX               ; load x coordinate
			 MOV    DX, 80               ; line length in bytes, here 80
			 MUL    DX
			 POP    BX                   ; BX <- y
			 PUSH   BX
						     ; loads y coordinate
			 MOV    CL, BL               ; saves low byte for shift
			
			 SHR    BX, 1                ; divide x by 8
			 SHR    BX, 1
			 SHR    BX, 1
			 ADD    BX, AX               ; adds the offset resulting from the MUL

			 AND    CL, 7                ; calculates the binary mask from x
			 XOR    CL, 7
			 MOV    AH, 1
			 SHL    AH, CL

			 MOV    DX, 3CEh             ; accessing video controller
			 MOV    AL, 8                ; loads binary mask into masking register
			 OUT    DX, AX

			 MOV    AX, (02h shl 8) + 5  ; write mode 2 &
			 OUT    DX, AX               ; read mode 0

			 MOV    AX, 0A000h           ; loads video segment
			 MOV    ES, AX              

			 MOV    AL, ES:[BX]          ; loads the latch register


			 MOV    AL, Color                ; set the color
			 MOV    ES:[BX], AL          ; rewrites the latch register

				; Resets the default values
				; into the graphic controller registers
  
			 MOV    AX, (0FFh shl 8 ) + 8
			 OUT    DX, AX

			 MOV    AX, (00h shl 8) + 5
			 OUT    DX, AX

			 POP    DX
			 POP    ES
			 POP    CX

			 RET

	   PutPixelProc  EndP

	   GetPixelProc  Proc NEAR
			 PUSH   DX
			 PUSH   CX

			 MOV    AX, DX     ; loads y coordinate
			 MOV    DX, 80     ; Multiplies by line width
			 MUL    DX
			 MOV    SI, CX     ; loads X coordinate
		  
			 SHR    SI, 1      ; Divides X by eight (2^3)
			 SHR    SI, 1
			 SHR    SI, 1
			 ADD    SI, AX     ; Adds offset from multiplication

			 AND    CL, 7      ; Calculates the binary mask from X
			 XOR    CL, 7
			 MOV    CH, 1
			 SHL    CH, CL

			 MOV    AX, 0A000h  ; loads ES with the video segment
			 MOV    ES, AX      

			 MOV    DX, 03CEh            ; Access the graphical controller
			 MOV    AX, (3 shl 8)+ 4     ; reads plane #3
			 XOR    BL, BL                                                     ;

		 GP1  :  OUT    DX, AX      ; Sets the plan to read
			 MOV    BH, ES:[SI] ; reads the latch register
			 AND    BH, CH      ; masks non interesting pixels
			 NEG    BH          
			 ROL    BX, 1     ; rotates bit 7 of BH into bit 1 of BL

			 DEC    AH        ; continues on the next plane
			 JGE    GP1       ; is it zero, if no ---> carry on

			 MOV    AL, BL    ; sends back the result into AL
			 MOV    AH, 00    ; makes sure the whole AX contains the color

			 POP    CX
			 POP    DX
			 RET

	  GetPixelProc   EndP


      CheckEntry: PUSHF              ; saves flags to keep them away from CMP effects

		  MOV    color, AL
		  PUSH   SI
		  MOV    BH, 0
		  MOV    BL, AL
		  MOV    SI, BX
		  MOV    AL, WinPalette[SI]
		  POP    SI

				       
		  CMP    AH, 12h     ; is this a presence test ?
		  JE     Pres_Test

		  CMP    AH, 00h     ; may be it is a Put Pixel request ?
		  JE     Put_Pixel

		  CMP    AH, 01h
		  JE     xorPut_Pixel

		  CMP    AH, 02h
		  JE     Get_Pixel   ; or a Get Pixel request 

		  CMP    AH, 03h     ; or even e Draw Box request !
		  JE     DrawBox 

		  POPF               ; but what the hell could it be else ?! 
		  RET    2     



     Pres_Test  : POPF
		  STI
		  MOV    AX, 2100h       ; Send back gboost signature
		  RET    2               ; done.

     Put_Pixel  : POPF
		  MOV    Color, AL
		  CALL   PutPixelProc
		  RET    2


     Get_Pixel:   POPF
		  XCHG   CX, DX
		  CALL   GetPixelProc
		  
		  PUSH   SI
		  MOV    BH, 0
		  MOV    BL, AL
		  MOV    SI, BX
		  MOV    AL, WinPalette[SI]
		  POP    SI

		  RET    2

   xorPut_Pixel:  POPF
		  XCHG   CX, DX
		  CALL   GetPixelProc  ; gets the current pixel color
		  XCHG   CX, DX
		  
		  MOV    BL, AL
		  XOR    Color, BL        ; XORs it with the requested color

		  CALL   PutPixelProc  ; draws the pixel with the resulting color
		  RET    2

   DrawBox      : 
		  MOV    X2, SI
		  MOV    Y2, DI
		  MOV    Color, AL
		  MOV    X1, DX 
		  MOV    Y1, CX

      Continue :  
			 PUSH   CX
			 PUSH   ES
			 PUSH   DX

			 MOV    AX, CX               ; load x coordinate
			 MOV    DX, 80               ; line length in bytes, here 80
			 MUL    DX
			 POP    BX                   ; BX <- y
			 PUSH   BX
						     ; loads y coordinate
			 MOV    CL, BL               ; saves low byte for shift
			
			 SHR    BX, 1                ; divide x by 8
			 SHR    BX, 1
			 SHR    BX, 1
			 ADD    BX, AX               ; adds the offset resulting from the MUL

			 AND    CL, 7                ; calculates the binary mask from x
			 XOR    CL, 7
			 MOV    AH, 1
			 SHL    AH, CL

			 MOV    DX, 3CEh             ; accessing video controller
			 MOV    AL, 8                ; loads binary mask into masking register
			 OUT    DX, AX

			 MOV    AX, (02h shl 8) + 5  ; write mode 2 &
			 OUT    DX, AX               ; read mode 0

			 MOV    AX, 0A000h           ; loads video segment
			 MOV    ES, AX              

			 MOV    AL, ES:[BX]          ; loads the latch register


			 MOV    AL, Color                ; set the color
			 MOV    ES:[BX], AL          ; rewrites the latch register

				; Resets the default values
				; into the graphic controller registers
  
			 MOV    AX, (0FFh shl 8 ) + 8
			 OUT    DX, AX

			 MOV    AX, (00h shl 8) + 5
			 OUT    DX, AX

			 POP    DX
			 POP    ES
			 POP    CX
      
		  CMP    DX, X2          ; is this the end of line
		  JE     NextLine        ; if yes, go to next line
		  INC    DX              ; if not process the next pixel
		  JMP    Continue        ; on the same line
       NextLine:  MOV    DX, X1          ; restart from line beginning
		  CMP    CX, Y2          ; is this the last line
		  JE     EndBox          ; if yes, done
		  INC    CX              ; if not go on the next line
		  JMP    Continue

      EndBox:     POPF

		  RET    2

      TSRint32  endp


   EndOfTSRCode:              

	Install:  MOV    AX, CS
		  MOV    DS, AX

		  MOV    DX, offset Welcome  ; .Welcoming message
		  MOV    AH, 09h
		  INT    21h


		  MOV    AH, 12h           ; .is it installed ?
		  INT    32h       
		  CMP    AH, 21h
		  JE     AlreadyInstErr

		  MOV    AX, 2532h           ; Revector INT 32 to our gBoost
		  MOV    DX, offset TSRint32
		  INT    21h
			   

       OkGoAhead: MOV    DX, offset InstSucc     ; Install Ok message 
		  MOV    AH, 09h
		  INT    21h

		  MOV    DX, EndOfTSRCode-Start+100h+15 ; Calculation of the
							; bytes number
		  MOV    Cl, 4
		  SHR    DX, CL        ; Converts into paragraphs

		  MOV    AL, 0         ; Exit code, 0 => No errors.
		  MOV    AH, 31h       ; Leave it resident  fn. 31h
		  INT    21h

							
		  
 AlreadyInstErr:  MOV    DX, offset AlreadyIn ; Already installed message
		  MOV    AH, 09h
		  INT    21h

		  MOV    AH, 4Ch
		  MOV    AL, 0FFh ; Exit code, -1 => errors occured
		  INT    21h


 code ENDS
  end Start



